# xDevs.com Python 10V test for 3458A
# http://xdevs.com/guide/ni_gpib_rpi/
import os.path
import sys
import Gpib
import time
import ftplib
import numbers
import signal
from Adafruit_BME280 import *

root_dir = '/repo/3458/'
fileName2002 = root_dir + 'na_2002_nplc10_tin_00000.csv'

cnt = 0
tread = 20
reflevel = 0.001

exttemp = 25.0
rh = 50.0
pascals = 100000.00
hectopascals = 1000.00

class Timeout():
  """Timeout class using ALARM signal"""
  class Timeout(Exception): pass

  def __init__(self, sec):
    self.sec = sec

  def __enter__(self):
    signal.signal(signal.SIGALRM, self.raise_timeout)
    signal.alarm(self.sec)

  def __exit__(self, *args):
    signal.alarm(0) # disable alarm

  def raise_timeout(self, *args):
    raise Timeout.Timeout()

class scpi_meter():
    temp = 38.5
    data = ""
    ppm = 0.0
    ampc = 0.0
    status_flag = 1
    temp_status_flag = 1
    global exttemp
    global rh
    global hectopascals

    def __init__(self,gpib,reflevel,name):
        self.gpib = gpib
        self.inst = Gpib.Gpib(0,self.gpib, timeout=60) # SCPI GPIB Address = self.gpib
        self.reflevel = reflevel
        self.name = name
        self.init_inst()

    def init_inst(self):
        # Setup SCPI DMM
        self.inst.clear()
	self.inst.write("*RST")
	self.inst.write("*CLR")
        self.inst.write(":SYST:AZER:TYPE SYNC")
        self.inst.write(":SYST:LSYN:STAT ON")
        self.inst.write(":SENS:FUNC 'VOLT:DC'")
	self.inst.write(":SENS:VOLT:DC:DIG 9; NPLC 10; AVER:COUN 3; TCON REP")
        self.inst.write(":SENS:VOLT:DC:AVER:STAT ON")
        self.inst.write(":SENS:VOLT:DC:RANG 20")
        self.inst.write(":FORM:ELEM READ")
        #self.inst.write(":DISP:WIND:TEXT:DATA \"               \";STAT ON;")
        #self.inst.write(":DISP:WIND2:TEXT:DATA \"               \";STAT ON;")
        #kei.write("READ?")

    def read_data(self,cmd):
        data_float = 0.0
        data_str = ""
        self.inst.write(cmd)
        try:
            with Timeout(20):
                data_str = self.inst.read()
        except Timeout.Timeout:
            print ("Timeout exception from dmm %s on read_data() inst.read()\n" % self.name)
            return (0,float(0))
        #print ("Reading from dmm %s = %s" % (self.name,data_str))
        try:
            data_float = float(data_str)
        except ValueError:
            print("Exception thrown by dmm %s on read_data() - ValueError = %s\n" % (self.name,data_str))
            return (0,float(0)) # Exception on float conversion, 0 = error
        return (1,data_float) # Good read, 1 = converted to float w/o exception

    def get_data(self):
        self.status_flag,data = self.read_data("READ?")
        if (self.status_flag):
            self.data = data
            #self.ppm = ((float(self.data) / self.reflevel)-1)*1E6
	    self.ampc = data
        return self.data

    def get_data_status(self):
        return self.status_flag

    def write_data(self,fileHandle):
        print time.strftime("%d/%m/%Y-%H:%M:%S;") + ("\033[1;31m[%8d]: %2.8f , dev %4.2f ppm,\033[1;39m  EXT_T:%3.2f , RH:%3.2f , Press:%4.2f hPa" % (cnt, float(self.data),float(self.ppm),float(exttemp),float(rh),float(hectopascals) ) )
        fileHandle.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + ("%16.8f;%16.8f;%3.1f;%3.2f;%3.2f;%4.2f;\r\n" % (float(self.data),float(self.reflevel),float(self.ppm),float(exttemp),float(rh),float(hectopascals) ) ))

    def print_ppm(self):
        self.inst.write(":DISP:WIND2:TEXT:DATA \"%5.4f nA\";STAT ON;" % float(self.ampc))
	#self.inst.write(":DISP:WIND2:TEXT:DATA \"  \"")

# Read Temperature, Humidity, and Barometric Pressure from BME280
def get_THP():
    global exttemp
    global rh
    global pascals
    global hectopascals
    exttemp = sensor.read_temperature()
    rh = sensor.read_humidity()
    pascals = sensor.read_pressure()
    hectopascals = pascals / 100

# Check if file exists, if not create it and add header
def create_local_file(fileName):
    if (os.path.isfile(fileName) == False):
        with open(fileName, 'a') as o:
            o.write("date;hp3458a;level;temp;ppm_level;ext_temp;rh;pressure;\r\n")
            print ("file %s does not exist\r\n" % fileName) 
    else: 
        print ("file %s exists\r\n" % fileName)

meter2002 = scpi_meter(10,reflevel,"2002_00000") # Gpib 10

#meter1.get_temp()
#meter2001.get_temp()
#meter3.get_temp()

#Setup temp/humidity/pressure sensor BME280
# OSAMPLE = oversampling mode
sensor = BME280(mode=BME280_OSAMPLE_8)
get_THP() # Read Temp, RH, Pressure from sensor

# Create file for 1st 3458A and write header if needed
create_local_file(fileName2002)

while cnt <= 100000:
    cnt+=1
    with open(fileName2002, 'a') as o3: # Open file handles for both files

        get_THP() # Read Temp, RH, Press from sensor

        meter2002.get_data()
        if(meter2002.get_data_status()):
            meter2002.print_ppm()
	    meter2002.write_data(o3)
    
        o3.close()
